﻿/*
 * integration.s
 *
 *  Author: razvan
 */
 
  
#include <avr/io.h>
#include "defines.h"
#include "delay.s"
#include "adc_macro.S"
#include "helpers.S"

/*
RUNUP Expansion
----------------

16Mhz clock cycle
20us integration
1us VREF toggle switching

+------+---------------+--------------+--------------+-------+-------+----------------+
| Step |     Func      | IntegrateVFP | IntegrateVFN | VREFP | VREFN |      VIN       |
+------+---------------+--------------+--------------+-------+-------+----------------+
| 1.   | SWITCH_TOGGLE |            - |            - | -     | -     | ON(from above) |
| 2.   | DELAY         |           11 |            - | ON    | -     | ON             |
| 3.   | LOOP_IF_P     |           14 |            - | ON    | -     | ON             |
| ---  |               |              |              |       |       |                |
| 4.   | SWITCH VRN    |           16 |            - | ON    | -     | ON             |
| 5.   | DELAY         |            - |          308 | -     | ON    | ON             |
| 6.   | INCR          |            - |          310 | -     | ON    | ON             |
| 7.   | JMP_IF        |            - |          314 | -     | ON    | ON             |
| 8.   | RJMP          |            - |          316 | -     | ON    | ON             |
| 1.   | SWITCH_TOGGLE |            - |          320 | -     | ON    | ON             |
| 2.   | DELAY         |            - |          331 | -     | ON    | ON             |
| 3.   | LOOP_IF_P     |            - |          334 | -     | ON    | ON             |
| ---  |               |              |              |       |       |                |
| 4.   | SWITCH VRP    |            - |          336 | -     | ON    | ON             |
| 5.   | DELAY         |          308 |            - | ON    | -     | ON             |
| 6.   | INCR          |          310 |            - | ON    | -     | ON             |
| 7.   | JMP_IF        |          314 |            - | ON    | -     | ON             |
| 8.   | RJMP          |          316 |            - | ON    | -     | ON             |
| 1.   | SWITCH_TOGGLE |          320 |            - | ON    | -     | ON             |
| 2.   | DELAY         |          331 |            - | ON    | -     | ON             |
| 3.   | LOOP_IF_P     |          334 |            - | ON    | -     | ON             |
| ---  |               |              |              |       |       |                |
| 4.   | SWITCH VRN    |          336 |            - | ON    | -     | ON             |
| 5.   | DELAY         |            - |          308 | -     | ON    | ON             |
| 6.   | INCR          |            - |          310 | -     | ON    | ON             |
| 7.   | JMP_IF        |            - |          314 | -     | ON    | ON             |
| 8.   | RJMP          |            - |          316 | -     | ON    | ON             |
+------+---------------+--------------+--------------+-------+-------+----------------+


*/

.global MEASURE
MEASURE:
    // r24:r25 -> timeout
    PUSHA
    sts OCR1AH, r25
    sts OCR1AL, r24
    // Setup constant registers
    eor ZERO, ZERO
    eor ONE, ONE
    inc ONE
    AUTO_ZERO:
        // Check if we have a command, if not go next
        JUMP_IF IS_ONE(UCSR0A, RXC0), exit
        READ_ADC ADC_AUTO_ZERO_READING

    REGISTER_CLEAR:
        .irp param,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31
            eor \param,\param
        .endr

        ldi SWITCH_TOGGLE_MASK, SWITCH_VREF_P | SWITCH_VREF_N
        // Start with VREF_P
        ldi SWITCH_TOGGLE_CURRENT, SWITCH_VREF_P

        // Stop the timer
        sts TCCR1B, ZERO
        // Clear the count
        sts TCNT1H, ZERO
        sts TCNT1L, ZERO
        // Clear the overflow flag
        ldi TEMP_REGISTER_ONE, (1 << OCF1A)
        out _SFR_IO_ADDR(TIFR1), TEMP_REGISTER_ONE
        // Prepare to start the timer again
        //  prescale of 64
        ldi TEMP_REGISTER_ONE, (1 << WGM12) | (1 << CS11) | (1 << CS10)
        // Start the timer for compare
        sts TCCR1B, TEMP_REGISTER_ONE
        // By this time we have 32+14 clock cycles for the REGISTER_CLEAR 'method'
    RUNUP:
        SWITCH SWITCH_VIN                                           //2 clock cycles
        //----------------- We are counting clocks from here down
        // Clock cycle accurate delay
        DELAY (INTEGRATE_TP/2)-4 // compensate for the next switch
        RUNUP_LOOP:
            SWITCH_TOGGLE SWITCH_TOGGLE_CURRENT, SWITCH_TOGGLE_MASK, SWITCH_VIN   //4 clock cycles
            //-------------------- From here we are integrating VIN + VRP as start
            //  the next loop we are integrating VIN+VRN and toggle on a subsequent loop
            DELAY us_to_cycles(1)-5 // 3 for jmp and 2 for switch
            // If integrator is positive go to the needed slope
            LOOP_IF_POSITIVE 1f                                     //3 clock cycles
       
            // The integrator is negative, we integrate VREFN to go positive
            SWITCH SWITCH_VREF_N | SWITCH_VIN                       //2 clock cycles
            //---------- From here we are integrating VREFN+VIN    
            DELAY us_to_cycles(20)-12
            INCR SLOPE_COUNT_VRN                                    //2 clock cycles
            JUMP_IF IS_ONE(TIFR1, OCF1A), PRERUNDOWN                //4 clock cycles 
            rjmp RUNUP_LOOP                                         //2 clock cycles

            // The integrator is positive, we integrate VREFP to go negative
        1:  SWITCH SWITCH_VREF_P | SWITCH_VIN                       //2 clock cycles
            //---------- From here we are integrating VREFP+VIN
            DELAY us_to_cycles(20)-12
            INCR SLOPE_COUNT_VRP                                    //2 clock cycles
            JUMP_IF IS_ONE(TIFR1, OCF1A), PRERUNDOWN                //4 clock cycles 
            rjmp RUNUP_LOOP                                         //2 clock cycles
    PRERUNDOWN:
        // When we exit the runup loop we exit with 6 missing clock cycles from a full integration slope period
        //  2 for rjmp and 4 for the new switch toggle compensate for that in the switch and extra delay
        DELAY 4
        // Make sure we are above 0
        // Integrate -VREF So we go in a positive direction until we go above zero
        SWITCH SWITCH_VREF_N                                        //2 clock cycles
        // -------------------- INTEGRATE VRN
    1:  INCR SLOPE_COUNT_PRERUNDOWN                                 //1 clock cycle 
        DELAY (PRERUNDOWN_TP - 4) //Extra clock cycles              // Extra delay
        LOOP_IF_NEGATIVE 1b                                         //3 clock cycles
        // Keep in mind that PRERUNDOWN has 2 extra clock cycles of integration
        //  before we do the switch
    RUNDOWN:
        // Faster LOOP, it takes only 4 clock cycles as opposed to 8 for the RUNUP slopes
        //      depending on the INTEGRATE_TP and RUNDOWN_TP at most it can be twice as fast
        // Go negative with VREF
        SWITCH SWITCH_VREF_P                                        //2 clock cycles
        // -------------------- INTEGRATE VRP
    1:  INCR SLOPE_COUNT_RUNDOWN_VRP                                //1 clock cycle 
        DELAY (RUNDOWN_TP - 4)
        LOOP_IF_POSITIVE 1b                                         //3 clock cycles 
        
        // Go positive with -VREF
        SWITCH SWITCH_VREF_N                                        //2 clock cycles
        // -------------------- INTEGRATE VRN
    1:  INCR SLOPE_COUNT_RUNDOWN_VRN                                //1 clock cycle
        DELAY (RUNDOWN_TP - 4)
        LOOP_IF_NEGATIVE 1b                                         //3 clock cycles

        // Go negative with VREF small
        SWITCH SWITCH_VREF_SMALL                                    //2 clock cycles
        // -------------------- INTEGRATE VRP SMALL
    1:  INCR SLOPE_COUNT_RUNDOWN_VRP_SMALL                          //1 clock cycle
        DELAY (RUNDOWN_TP - 5)
        LOOP_IF_POSITIVE 1b                                         //3 clock cycles

        // Hold to read ADC
        SWITCH SWITCH_NONE                                          //2 clock cycles

        READ_ADC ADC_RUNDOWN_READING                                //220 clock cycles for prescale of 16, and 16Mhz clock
    UART_SEND:
        // Finally send the UART DATA
        // For 16Mhz - 500k baud -> 32 clock cycles per bit
        //  send reg takes an additional 5 clock cycles at best
        //  send char takes an additional 6 clock cycles at best
        // TODO: Maybe it's faster to just calculate here what we need to send, and send only that, or part of the data anyway???
        UART_SEND_REG   ADC_AUTO_ZERO_READING
        UART_SEND_REG   SLOPE_COUNT_VRP
        UART_SEND_REG   SLOPE_COUNT_SWITCH_VRP_VRN
        UART_SEND_REG   SLOPE_COUNT_VRN
        UART_SEND_REG   SLOPE_COUNT_SWITCH_VRN_VRP
        UART_SEND_REG   SLOPE_COUNT_PRERUNDOWN
        UART_SEND_REG   SLOPE_COUNT_RUNDOWN_VRP
        UART_SEND_REG   SLOPE_COUNT_RUNDOWN_VRN
        UART_SEND_REG   SLOPE_COUNT_RUNDOWN_VRP_SMALL
        UART_SEND_REG   ADC_RUNDOWN_READING

        UART_SEND_CHAR '\n'
        jmp AUTO_ZERO
exit:
    POPA
    ret

